@@ -2,11 +2,15 @@ use strict;
 use warnings;
 package YAML::PP::Common;
 
-our $VERSION = '0.030'; # VERSION
+our $VERSION = '0.032'; # VERSION
 
 use base 'Exporter';
 
-our @EXPORT_OK = qw/
+my @p = qw/
+    PRESERVE_ALL PRESERVE_ORDER PRESERVE_SCALAR_STYLE PRESERVE_FLOW_STYLE
+    PRESERVE_ALIAS
+/;
+my @s = qw/
     YAML_ANY_SCALAR_STYLE YAML_PLAIN_SCALAR_STYLE
     YAML_SINGLE_QUOTED_SCALAR_STYLE YAML_DOUBLE_QUOTED_SCALAR_STYLE
     YAML_LITERAL_SCALAR_STYLE YAML_FOLDED_SCALAR_STYLE
@@ -17,10 +21,13 @@ our @EXPORT_OK = qw/
 
     YAML_ANY_MAPPING_STYLE
     YAML_BLOCK_MAPPING_STYLE YAML_FLOW_MAPPING_STYLE
-
-    PRESERVE_ALL PRESERVE_ORDER PRESERVE_SCALAR_STYLE PRESERVE_FLOW_STYLE
-    PRESERVE_ALIAS
 /;
+our @EXPORT_OK = (@s, @p);
+
+our %EXPORT_TAGS = (
+    PRESERVE => [@p],
+    STYLES => [@s],
+);
 
 use constant {
     YAML_ANY_SCALAR_STYLE           => 0,
@@ -216,14 +223,33 @@ YAML::PP::Common - Constants and common functions
 
 =head1 SYNOPSIS
 
+    use YAML::PP::Common ':STYLES';
+    # or
     use YAML::PP::Common qw/
         YAML_ANY_SCALAR_STYLE YAML_PLAIN_SCALAR_STYLE
         YAML_SINGLE_QUOTED_SCALAR_STYLE YAML_DOUBLE_QUOTED_SCALAR_STYLE
         YAML_LITERAL_SCALAR_STYLE YAML_FOLDED_SCALAR_STYLE
+        YAML_QUOTED_SCALAR_STYLE
+
+        YAML_ANY_SEQUENCE_STYLE
+        YAML_BLOCK_SEQUENCE_STYLE YAML_FLOW_SEQUENCE_STYLE
+
+        YAML_ANY_MAPPING_STYLE
+        YAML_BLOCK_MAPPING_STYLE YAML_FLOW_MAPPING_STYLE
     /;
 
+    use YAML::PP::Common ':PRESERVE';
+    # or
+    use YAML::PP::Common qw/
+        PRESERVE_ALL PRESERVE_ORDER PRESERVE_SCALAR_STYLE PRESERVE_FLOW_STYLE
+        PRESERVE_ALIAS
+    /:
+
 =head1 DESCRIPTION
 
+This module provides common constants and functions for modules working with
+YAML::PP events.
+
 =head1 FUNCTIONS
 
 =over
